import csv
try:
    from cStringIO import StringIO
except ImportError:
    try:
        from StringIO import StringIO
    except ImportError:
        from io import StringIO
try:
    import ujson
except ImportError:
    ujson = None
import json
try:
    import msgpack
except ImportError:
    msgpack = None


class GroongaResult(object):

    def __init__(self, data, output_type="json", encoding='utf-8'):
        self.raw_result = data
        if output_type == 'tsv':
            # TODO: not implement
            c = csv.reader(StringIO(data), delimiter='\t')
        elif output_type == 'msgpack':
            if msgpack:
                unpacker = msgpack.Unpacker()
                _result = msgpack.unpackb(data)
            else:
                raise Exception("msgpack is not support")
        elif output_type == 'json':
            if ujson and encoding == 'utf-8':
                _result = ujson.loads(data)
            else:
                _result = json.loads(data, encoding)
        else:   # xml or other types...
            # TODO: not implement
            pass
        self.status = _result[0][0]
        self.start_time = _result[0][1]
        self.elapsed = _result[0][2]
        if len(_result) == 1 and self.status != 0:
            self.body = _result[0][3]
        elif len(_result) == 1 and self.status == 0:
            self.body = ""  # handling invalid result
        else:
            self.body = _result[1]


class GroongaSelectResult(GroongaResult):

    def __init__(self, data, output_type="json", encoding='utf-8'):
        super(GroongaSelectResult, self).__init__(data, output_type, encoding)
        self.items = []
        self.hit_num = -1   # default is -1 (Error)
        if len(self.body) != 0:
            self.hit_num = self.body[0][0][0]
        if self.status == 0:
            keys = [k[0] for k in self.body[0][1]]
            self.items = [
                dict(list(zip(keys, item))) for item in self.body[0][2:]]



import sys
import struct
import unittest
try:
    from mock import patch, Mock
except ImportError:
    from unittest.mock import patch, Mock
from poyonga import Groonga, GroongaResult


class PoyongaHTTPTestCase(unittest.TestCase):

    def setUp(self):
        self.g = Groonga()

    @patch('poyonga.client.urlopen')
    def test_json_result_with_http(self, mock_urlopen):
        m = Mock()
        m.read.side_effect = ['[[0, 1337566253.89858, 0.000354], {}]']
        mock_urlopen.return_value = m
        ret = self.g.call('status')
        self.assertEqual(type(ret), GroongaResult)
        self.assertEqual(ret.status, 0)


class PoyongaGQTPTestCase(unittest.TestCase):

    def setUp(self):
        self.g = Groonga(protocol='gqtp')

    @patch('poyonga.client.socket.socket')
    def test_json_result_with_gqtp(self, mock_socket):
        m = Mock()
        if sys.version_info[0] == 3:
            _data = b"{}"
        else:
            _data = "{}"
        _proto, _qtype, _keylen, _level, _flags, _status, _size, _opaque, _cas = \
            0xc7, 0x02, 0, 0, 0, 0, 2, 0, 0
        packdata = struct.pack("!BBHBBHIIQ2s",
                               _proto, _qtype, _keylen, _level, _flags, _status, _size, _opaque, _cas, _data)
        m.recv.return_value = packdata
        mock_socket.return_value = m
        ret = self.g.call('status')
        self.assertEqual(type(ret), GroongaResult)
        self.assertEqual(ret.status, 0)


if __name__ == '__main__':
    unittest.main()
